package org.eclipse.e4.ui.css.swt.dom;

import org.eclipse.swt.graphics.Color;

public interface IHeaderCustomizationElement {

	public void setHeaderColor(Color color);

	public void setHeaderBackgroundColor(Color color);

}
