/**
 * Copyright (c) 2008, 2015 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *      IBM Corporation - initial API and implementation
 */
package org.eclipse.e4.ui.model.application.ui.basic.impl;

import org.eclipse.e4.ui.model.application.ui.basic.MWizardDialog;
import org.eclipse.emf.ecore.EClass;

/**
 * <!-- begin-user-doc --> An implementation of the model object '<em><b>Wizard
 * Dialog</b></em>'. <!-- end-user-doc -->
 *
 * @since 1.1
 * @generated NOT
 * @deprecated
 * @noreference This class is not intended to be referenced by clients.
 * @see <a href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=531054">Bug
 *      531054</a>
 */
@Deprecated
public class WizardDialogImpl extends DialogImpl implements MWizardDialog {
	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 * @deprecated
	 * @noreference This method is not intended to be referenced by clients.
	 * @see <a href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=531054">Bug
	 *      531054</a>
	 */
	@Deprecated
	protected WizardDialogImpl() {
		super();
	}

	/**
	 * <!-- begin-user-doc --> <!-- end-user-doc -->
	 * 
	 * @generated NOT
	 * @deprecated
	 * @noreference This method is not intended to be referenced by clients.
	 * @see <a href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=531054">Bug
	 *      531054</a>
	 */
	@Deprecated
	@Override
	protected EClass eStaticClass() {
		return BasicPackageImpl.Literals.WIZARD_DIALOG;
	}

} //WizardDialogImpl
