/**
 * Copyright (c) 2008, 2015 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *      IBM Corporation - initial API and implementation
 */
package org.eclipse.e4.ui.model.application.ui.basic;


/**
 * <!-- begin-user-doc --> A representation of the model object '<em><b>Wizard
 * Dialog</b></em>'. <!-- end-user-doc -->
 *
 * <!-- begin-model-doc -->
 * <p>
 * A subclass of Window representing a wizard dialog.
 * </p>
 * 
 * @since 1.1
 * @noimplement This interface is not intended to be implemented by clients.
 * @noreference This model element is still provisional. <!-- end-model-doc -->
 *
 * @model
 * @generated NOT
 * @deprecated
 * @see <a href="https://bugs.eclipse.org/bugs/show_bug.cgi?id=531054">Bug
 *      531054</a>
 */
@Deprecated
public interface MWizardDialog extends MDialog {
} // MWizardDialog
