package org.eclipse.e4.ui.workbench.addons.minmax;

import org.junit.runner.RunWith;
import org.junit.runners.Suite;
import org.junit.runners.Suite.SuiteClasses;

@RunWith(Suite.class)
@SuiteClasses({ MaximizeBugTest.class, MaximizePartSashContainerPlaceholderTest.class, MaximizableChildrenTag.class })
public class AllTests {

}
