<!doctype html public "-//w3c//dtd html 4.0 transitional//en">
<html>
<head>
   <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
   <title>Package-level Javadoc</title>
</head>
<body>
Provides the Tabbed Properties framework. This allows client views and editors to
provide a tabbed property sheet page to the Properties view. 
See TabbedPropertySheetPage.
The tabs and sections to show for the selected element(s) in the source part are
defined by extensions to this plug-in's extension points.
</body>
</html>
