/*******************************************************************************
 * Copyright (c) 2014 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.demo.cssbridge.ui;

public interface ICommandIds {
	interface CSSThemeSwitchCommands {
		String SWITCH_TO_EMPTY_CSS_THEME = "org.eclipse.e4.demo.cssbridge.switch.to.empty.css.theme";

		String SWITCH_TO_BLUE_CSS_THEME = "org.eclipse.e4.demo.cssbridge.switch.to.blue.css.theme";

		String SWITCH_TO_GREEN_CSS_THEME = "org.eclipse.e4.demo.cssbridge.switch.to.green.css.theme";

		String SWITCH_TO_RED_CSS_THEME = "org.eclipse.e4.demo.cssbridge.switch.to.red.css.theme";
	}
}
