Shell, SashForm, Button, Label {
	font-family: "Lucida Grande",Serif;
	font-size: 11;
	background: url(./images/osx_back.jpg);
}

ToolBar, CLabel {
	font-family: "Lucida Grande",Serif;
	font-size: 9;
	background: url(./images/osx_back.jpg);
}

Tree {
	font-family: "Lucida Grande",Serif;
	font-size: 11;
	background-color: lightgrey, white, white, lightgrey, 10%, 90% 100%;
}

CTabFolder:selected {
	background: url(./images/osx_tab.jpg);
	background-color: rgb(162,208,242);
	color: black;
}

CTabFolder {
	font-family: "Lucida Grande",Serif;
	font-size: 11;
	font-weight: normal;
	background-color: lightgrey;
	color: black;
}
