/*******************************************************************************
 * Copyright (c) 2013, 2015 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 ******************************************************************************/

package org.eclipse.e4.ui.internal.workbench.swt;

import org.eclipse.e4.ui.workbench.IPresentationEngine;

public class CSSConstants {
	public static final String CSS_BUSY_CLASS = "busy";

	public static final String CSS_ACTIVE_CLASS = IPresentationEngine.ACTIVE;

	public static final String CSS_NO_FOCUS_CLASS = "noFocus";

	public static final String CSS_HIGHLIGHTED_CLASS = "highlighted";

	public static final String CSS_CONTENT_CHANGE_CLASS = "contentChange";
}