/*******************************************************************************
 * Copyright (c) 2014 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.demo.cssbridge.ui;

public class CSSTheme {
	public static final String EMPTY_THEME_ID = "org.eclipse.e4.demo.cssbridge.empty.theme";

	public static final String BLUE_THEME_ID = "org.eclipse.e4.demo.cssbridge.blue.theme";

	public static final String GREEN_THEME_ID = "org.eclipse.e4.demo.cssbridge.green.theme";

	public static final String RED_THEME_ID = "org.eclipse.e4.demo.cssbridge.red.theme";
}
