/*******************************************************************************
 * Copyright (c) 2006 IBM Corporation and others.
 *
 * This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 ******************************************************************************/

package org.eclipse.ui.tests.dynamicplugins;

/**
 * DynamicTestType is a type used for contributing property pages in the
 * dynamic tests.
 * @since 3.2
 *
 */
public class DynamicTestType {

	/**
	 * Create a new instance of the receiver.
	 */
	public DynamicTestType() {
		super();
	}

}
